/*******************************************************************************
 * Main object that interacts between the user and the backend.
 ******************************************************************************/
var Newslive = {

  _queue: null,
  get queue() { return this._queue; },
  set queue(val) { this._queue = val; },
  
  _timer: null,
  get timer() { return this._timer; },
  set timer(val) { this._timer = val; },

  _obService: null,
  get obService() { return this._obService; },
  set obService(val) { this._obService = val; },
  
  _view: null,
  get view() { return this._view; },
  set view(val) { this._view = val; },
  
  _selection: null,
  get selection() { return this._selection; },
  set selection(val) { this._selection = val; },
  
  get menu() { return document.getElementById("newslive_menu"); },
      
  // nsISupports
  QueryInterface: function nl_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsIController) ||    
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  }, 
  
  // nsIObserver
  observe: function nl_observe(subject, topic, data) {
    this.queue.push({ "data": data, "subject": subject });
  },
  
  // nsIController
  supportsCommand: function nl_supportsCommand(id) {
    switch (id) {
    case "newslive_open":
    case "newslive_openWin":
    case "newslive_openTab":
    case "newslive_openSidebar":
    case "newslive_openAll":
    case "newslive_reload":    
    case "newslive_options":
      return true;
     default:
       return false;
    }
  },
  
  isCommandEnabled: function nl_isCommandEnabled(id) {
    var item = this.selection;
    var type = (!item) ? null : item.getAttribute("itemtype");
    switch (id) {
    case "newslive_open":
      return (type == "item");
    case "newslive_openWin":
      return (type == "item");
    case "newslive_openTab":
      return (type == "item");    
    case "newslive_openSidebar":
      return (type == "item");      
    case "newslive_openAll":
      return (type == "feed");    
    case "newslive_reload": 
      return (type == "feed");       
    case "newslive_options":
      return (type !== null);
    default:
      return false;
    }
  },
  
  doCommand: function nl_doCommand(id) {
    var item = this.selection;
    switch (id) {
    case "newslive_open":
      this.view.open(item, "current");
      break;  
    case "newslive_openWin":
      this.view.open(item, "window");
      break;      
    case "newslive_openTab":
      this.view.open(item, "tab");
      break;
    case "newslive_openSidebar":
      this.view.open(item, "sidebar");
      break;      
    case "newslive_openAll":
      this.view.openAll(item);
      break;
    case "newslive_reload": 
      this.view.reload(item);
      break;    
    case "newslive_options":
      this.view.utils.openOptions();
      break;
    default:
      return;
    }
  },
  
  onEvent: function nl_onEvent(name) {
    goUpdateCommand("newslive_open");    
    goUpdateCommand("newslive_openWin");
    goUpdateCommand("newslive_openTab"); 
    goUpdateCommand("newslive_openSidebar"); 
    goUpdateCommand("newslive_openAll");  
    goUpdateCommand("newslive_reload");  
    goUpdateCommand("newslive_options");       
  },
      
  onLoad: function nl_onLoad() {   
  
    /** 
     * setup an event queue.  When we get notification from the back-end
     * An event will be pushed onto the queue.  The queue is then checked
     * every 500 milliseconds and is used to update the front-end.  This
     * is done so we disconnect the calls and keep the front-end responsive.
     */
    var self = this;
    this.queue = [];
    this.timer = window.setInterval(function() { self.checkQueue(); }, 500);

    // setup the util object
    var parentNode = document.getElementById("newslive_menupopup");
    var lastChild = document.getElementById("newslive_menuseparator");
    this.view = new NewsliveView("menu", parentNode, lastChild);
    
    // hookup this as a controller
    window.controllers.appendController(this);

    // get the observer service and add ourself as an observer
    this.obService = Cc["@mozilla.org/observer-service;1"].
                     getService(Ci.nsIObserverService);
    this.obService.addObserver(this, "netscape-newslive", false);
    
    // hookup the menu listener
    this.view.parentNode.addEventListener("DOMMenuItemActive", function(event) { 
                                          self.setSelection(event); }, true);
    this.view.parentNode.addEventListener("mousedown", function(event) { 
                                          self.setSelection(event); }, true);
    
    // load the menu                               
    window.setTimeout(function() { self.loadDelayed(); }, 0);
  },
  
  onUnload: function nl_onUnload() {
  
    // stop the queue timer
    window.clearInterval(this.timer);
    
    // remove ourself as an observer
    this.obService.removeObserver(this, "netscape-newslive");
    
    // unhook the controller
    window.controllers.removeController(this);
    
    // clean up the utility object
    this.view.parentNode = null;
    this.view.lastChild = null;
    this.view.type = null;

    // remove variables
    this.queue = null;
    this.timer = null; 
    this.obService = null;
    this.view = null;    
  },
  
  checkQueue: function nl_checkQueue() {   
    while (this.queue.length) {
      var item = this.queue.shift();
      this[item.data](item.subject);    
    }
  },
      
  loadDelayed: function nl_loadDelayed() {   
    this.view.loadDelayed();
  },

  sameFeeds: function nl_sameFeeds(service) {},
  newFeeds: function nl_newFeeds(service) {
    this.loadDelayed();
  },
  
  changedFeeds: function nl_changedFeeds(service) {
  
    // we are not the top window
    if (getTopWin() != window)
      return;
    
    // see if we need to ask
    var branch = this.view.utils.branch;  
    var dontAsk = !branch.getBoolPref("showChanged");
    if (!dontAsk)  {

      // show changed prompt
      var title = this.view.utils.getString("newslive.overlay.changed.title");
      var message = this.view.utils.getString("newslive.overlay.changed.message");
      var show = this.view.utils.getString("newslive.overlay.changed.show");
    
      var prompt = Cc["@mozilla.org/embedcomp/prompt-service;1"].
                   getService(Ci.nsIPromptService);
      var checkbox = { value: false };
      
      // not open
      var open =  prompt.confirmCheck(window, title, message, show, checkbox);
      if (!open)
        return;
        
      // save the checked value
      branch.setBoolPref("showChanged", !checkbox.value);
    }
    
    // show the manage dialog
    this.view.utils.openManage();
  },
  
  addFeed: function nl_addFeed(feed) { this.view.addFeed(feed); },
  removeFeed: function nl_removeFeed(feed) { this.view.removeFeed(feed); },
  updateFeed: function nl_updateFeed(feed) { this.view.updateFeed(feed); },  
  addItem: function nl_addItem(item) { this.view.addItem(item); },
  removeItem: function nl_removeItem(item) { this.view.removeItem(item); },
  updateItem: function nl_updateItem(item) { this.view.updateItem(item); },
  addItems: function nl_addItems(items) { this.view.addItems(items); },
  removeItems: function nl_removeItems(feed) { this.view.removeItems(feed); }, 

  setSelection: function nl_setSelection(event) {
    this.selection = event.target;
    goOnEvent(window, 'select');
    event.stopPropagation();    
  },
  
  buildContextMenu: function nl_buildContextMenu(event) {
  
    // setup variables
    var separator = null;
    var command = null;
    var visibleItemsBeforeSep = false;
    var anyVisible = false;
    
    // loop through the context menu
    var popup = event.target;
    for (var i=0; i<popup.childNodes.length; ++i) {
    
      // get the menuitem
      var item = popup.childNodes[i];
      
      // determine if item should be visible
      if (item.localName != "menuseparator") {
        command = document.getElementById(item.getAttribute("command"));
        item.hidden = command.hasAttribute("disabled");

        // item is visible mark variables as such
        if (!item.hidden) {
          visibleItemsBeforeSep = true;
          anyVisible = true;
 
          // show the separator above the menu-item if any
          if (separator) {
            separator.hidden = false;
            separator = null;
          }
        }
      }

      // menuseparator
      else {

        // initially hide it. It will be unhidden if there will be at least one
        // visible menu-item above and below it.
        item.hidden = true;

        // we won't show the separator at all if no items are visible above it
        if (visibleItemsBeforeSep)
          separator = item;
 
        // new separator, count again:
        visibleItemsBeforeSep = false;
      }
    }

    return anyVisible;
  }
};

window.addEventListener("load", function (event) { Newslive.onLoad(); }, false);
window.addEventListener("unload", function(event) { Newslive.onUnload(); }, false);