/*******************************************************************************
 * Main object that interacts between the user and the backend.
 ******************************************************************************/
var Sitemail = {
  
  // nsISupports
  QueryInterface: function sml_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
  
  observe: function sml_observe(aSubject, aTopic, aData) {
  
    // ignore other topics
    if (aTopic != "netscape-sitemail")
      return;

    // update   
    var self = this;
    window.setTimeout(function() { self.update(); }, 0); 
  },
  
  _obService: null,
  get obService() { return this._obService; },
  set obService(val) { this._obService = val; },
  
  _smlService: null,
  get smlService() { return this._smlService; },
  set smlService(val) { this._smlService = val; },

  onLoad: function sml_onLoad() {
    
    // Get the observer service and add ourself as an observer
    this.obService = Cc["@mozilla.org/observer-service;1"].
                     getService(Ci.nsIObserverService);
    this.obService.addObserver(this, "netscape-sitemail", false);
    
    // Get the sitemail service
    this.smlService = Cc["@netscape.com/sitemail/service;1"].
                      getService(Ci.nsISitemailService);    
      
    // Update the UI
    var self = this;
    window.setTimeout(function() { self.update(); }, 100);                      
  },
  
  onUnload: function sml_onUnload() {
    
    // remove ourself as an observer
    this.obService.removeObserver(this, "netscape-sitemail");

    // remove variables
    this.obService = null;
    this.smlService = null;     
  },
  
  getString: function sml_getString(name, replace) { 
  
    // get the string bundle
    var bundle = document.getElementById("sitemail_bundle");
    if (!bundle)
      return null;
    
    // return the string  
    if (!replace)
      return bundle.getString(name);
    else
      return bundle.getFormattedString(name, replace);
  },
      
  update: function sml_update() {

    // Get the sitemail Button
    var button = document.getElementById("sitemail_button");
    if (!button)
      return;
    
    // Update with not signed in
    const SML_PREFIX = "sitemail.overlay.tooltip.";
    if (!this.smlService.isSignedIn) {
      button.removeAttribute("new");
      button.setAttribute("tooltiptext", this.getString(SML_PREFIX + "login"));
      return;
    }
    
    // Update with no activity
    if (!this.smlService.hasMail) {
      button.removeAttribute("new");
      button.setAttribute("tooltiptext", this.getString(SML_PREFIX + "none"));
      return;
    }
    
    // Update with activity
    var count = String(this.smlService.totalCount);
    button.setAttribute("new", "true");
    if (count == "1")
      button.setAttribute("tooltiptext", this.getString(SML_PREFIX + "single", [count]));
    else
      button.setAttribute("tooltiptext", this.getString(SML_PREFIX + "plural", [count]));
    
  },
  
  open: function sml_open(event) {
    var url = this.smlService.URL;
    openUILink(url, event);
  }
};

window.addEventListener("load", function (event) { Sitemail.onLoad(); }, false);
window.addEventListener("unload", function(event) { Sitemail.onUnload(); }, false);